import os
import logging
from tempfile import mkdtemp, mktemp
from django.core import management
from fabric.api import run, sudo, local, put, env, settings, abort, cd
from fabric.contrib.console import confirm
from fabric.contrib.files import append, contains
from fabric.utils import abort
from deployment import (settings_staging,
                        settings_production,
                        settings_testing)


VALID_DEPLOYMENT_TYPES = ['production',
                          'staging',
                          'testing']

env.user = 'ubuntu'
env.key_filename = '/home/ubuntu/alesage_lcy01.key'

def apt_get_everything():
    """apt-get everything required to be a qa_dashboard."""
    sudo("apt-get install -y bzr apache2 libapache2-mod-wsgi \
          python-django postgresql python-psycopg2 python-pip python-tz \
          python-django-auth-openid django-tables python-django-celery \
          python-numpy python-yaml python-mock python-django-south \
          python-tastypie")


def restart_postgres():
    """Restart our fav DB server."""
    sudo("service postgresql restart")


def ubuntu_postgres_user_exists():
    """Does the postgres user exist and can I have her phone number?"""
    result = sudo("psql postgres -tAc \"SELECT 1 FROM pg_roles WHERE rolname='ubuntu'\"",
                  user='postgres')
    return int(result) == 1


def create_ubuntu_postgres_user():
    """Create 'ubuntu' postgres user."""

    with settings(warn_only=True):
        sudo("createuser -U postgres -s ubuntu -P", user='postgres')
        sudo("createdb ubuntu", user='postgres')


def database_exists(name='qa_dashboard'):
    """Does the database exist?"""
    result = sudo("psql -tAc \"SELECT * FROM pg_database WHERE datname='{}';\"".format(name),
                  user='postgres')
    return result != ""


def destroy_database(name='qa_dashboard'):
    """Destroy postgresql qa_dashboard database."""
    with settings(warn_only=True):
        # we could warn on this if it doesn't exist (but we don't expect to)
        sudo("dropdb {}".format(name),
             user='postgres')


def create_database(name='qa_dashboard'):
    """Create a fresh db for user ubuntu."""
    sudo("createdb -E utf8 -O ubuntu {} -T template0".format(name),
         user='postgres')


def adjust_pg_hba_conf():
    """We need md5 password checking for localhost."""
 
    # TODO: this sed dinna work under fabric WTF:
    # sudo sed 's/^\(local\s*all\s*all\s*\)\(.*\)$/\1md5/' /etc/postgresql/9.1/main/pg_hba.conf
    # . . . so we're just copying a baked file
    put("./deployment/pg_hba.conf", "/etc/postgresql/9.3/main/", use_sudo=True)
    restart_postgres()


def setup_database(name='qa_dashboard'):
    """Create postgresql qa_dashboard user and database.

    Name defaults to 'qa_dashboard' for production, suggest
    'qa_dashboard-staging' for staging.
    """
    adjust_pg_hba_conf()
    if not ubuntu_postgres_user_exists():
        create_ubuntu_postgres_user()
    if database_exists(name):
        destroy_database(name)
    create_database(name)
    restart_postgres()


def checkout_qa_dashboard(install_dir, branch="lp:helipad"):
    """Checkout branch into 'qa_dashboard' in the user's home dir.

    Bazaar and fabric don't get along so we can't just bzr branch,
    instead cp from a local checkout.
    """
    # blow away whatever's there at the install path
    sudo("rm -rf {}/*".format(install_dir))
    sudo("mkdir -p {}".format(install_dir))
    sudo("chown ubuntu {}".format(install_dir))

    # get a local checkout of the source, pack, and ship
    tmp_dir = mkdtemp()
    tmp_install_dir = os.path.join(tmp_dir, 'qa_dashboard')
    local("bzr branch {} {}".format(branch, tmp_install_dir))
    # this is kind-of silly but the tar -C option wasn't working with globbing
    orig_dir = os.getcwd()
    os.chdir(tmp_install_dir)
    local("tar --exclude fabfile.pyc --overwrite -czvf /tmp/qa_dashboard-source.tar.gz *")
    os.chdir(orig_dir)
    put("/tmp/qa_dashboard-source.tar.gz",
        install_dir,
        use_sudo=True)
    with cd(install_dir):
        run("tar -xvf qa_dashboard-source.tar.gz")
    run("rm {}".format(os.path.join(install_dir, "qa_dashboard-source.tar.gz")))
    local("rm -rf {}".format(tmp_install_dir))

    # set perms so that www-data can access
    sudo("chown -R ubuntu:qa_dashboard {}".format(install_dir))
    sudo("chmod -R g+rw {}".format(install_dir))


def restart_apache():
    """Restart our fav webserver."""
    sudo("service apache2 restart")


def configure_apache(apache_config_filepath):
    """Configure apache2: copy our site config, set up perms, etc.

    'deployment/qa_dashboard-staging', e.g.
    """
    basename = os.path.basename(apache_config_filepath)
    sudo("cp {} /etc/apache2/sites-available/{}.conf".format(
            apache_config_filepath,
            basename))
    sudo("adduser www-data qa_dashboard")
    # we'll take the last term of the apache config path as the site name to enable
    site_name = os.path.split(apache_config_filepath)[-1]
    sudo("a2ensite {}".format(site_name))
    restart_apache()


def setup_users():
    """Create a qa_dashboard group and add ubuntu to it.

    This facilitates apache access.
    """
    with settings(warn_only=True):
        sudo("addgroup qa_dashboard")
        sudo("adduser ubuntu qa_dashboard")


def copy_local_settings(install_dir,
                        settings_filename="settings_production.py"):
    """Copy settings into place."""
    sudo("cp {} {}".format(
            os.path.join(install_dir,
                         'deployment',
                         settings_filename),
            os.path.join(install_dir,
                         "qa_dashboard",
                         "local_settings.py")),
         user='ubuntu')


def collect_static_media(install_dir):
    """Use the Django utility to collect all of our static files."""

    with cd(install_dir):
        sudo("python manage.py collectstatic --noinput", user='ubuntu')


def setup_cron_job(install_dir):
    """Add our qa-dashboard-update.sh to crontab."""

    cron_job_line = "42 * * * * {}/scripts/qa-dashboard-update.sh".format(install_dir)
    tmp_cron_filename = mktemp()
    # our crontab might be empty; ignore that error
    with settings(warn_only=True):
        run("crontab -l 2>/dev/null > {}".format(tmp_cron_filename),)
    if contains(tmp_cron_filename, cron_job_line):
        print cron_job_line + " already in crontab."
    else:
        append(tmp_cron_filename, cron_job_line)
        run("crontab {}".format(tmp_cron_filename))
        run("rm {}".format(tmp_cron_filename))


def deploy(deployment_type, branch="lp:~helipad-team/helipad/coverage"):
    """Push this branch to the production site.

    deployment_type should be one of 'production', 'staging', or 'testing'.
    """

    if deployment_type not in VALID_DEPLOYMENT_TYPES:
        abort("Deployment must be one of {}.".format(VALID_DEPLOYMENT_TYPES))
    install_dir = {'production': settings_production.INSTALL_PATH,
                   'staging': settings_staging.INSTALL_PATH,
                   'testing': settings_testing.INSTALL_PATH}[deployment_type]
    checkout_qa_dashboard(install_dir, branch)
    apache_config_filename = {'production': 'qa_dashboard',
                              'staging': 'qa_dashboard-staging',
                              'testing': 'qa_dashboard-testing'}[deployment_type]
    configure_apache(
        os.path.join(install_dir,
                     'deployment',
                     apache_config_filename))
    setup_cron_job(install_dir)
    # compose a settings_filename, e.g. settings_testing.py
    settings_filename = "settings_{}.py".format(deployment_type)
    copy_local_settings(install_dir,
                        settings_filename)
    collect_static_media(install_dir)
    with cd(install_dir):
        run("python manage.py syncdb")
        run("python manage.py migrate")


def setup(deployment_type, branch="lp:~helipad-team/helipad/coverage"):
    """Setup the qa_dashboard site on a new server.

    deployment_type should be one of 'production', 'staging', or 'testing'.

    NOTE: DROPS DB.
    """

    if deployment_type not in VALID_DEPLOYMENT_TYPES:
        abort("deployment_type must be one of {}.".format(VALID_DEPLOYMENT_TYPES))
    apt_get_everything()
    setup_users()
    database_name = {'production': 'qa_dashboard',
                     'staging': 'qa_dashboard-staging',
                     'testing': 'qa_dashboard-testing'}[deployment_type]
    setup_database(database_name)
    deploy(deployment_type, branch=branch)
