"""Create/Update jenkins ci and autolanding jobs for a given stack

- Reads stack configuration from YAML configuration file
- Create/updates the jenkins jobs on the server configured in the credentials
file

"""
# Copyright (C) 2013, Canonical Ltd (http://www.canonical.com/)
#
# Author: Francis Ginther <francis.ginther@canonical.com>
#
# This software is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

import copy
import logging
import os

from c2dconfigutils import (
    dict_union,
    get_ci_base_job_name,
    load_default_cfg,
    load_stack_cfg,
    set_logging,
)
from cu2dUpdateCi import UpdateCi

from gaps.util import add

logger = logging.getLogger('qa_dashboard')
job_names = []


class OutputCi(UpdateCi):
    def generate_jobs(self, job_list, project_name, job_type, build_type,
                      job_config, job_template, build_template, job_data):
        """ Generates the main job and builder jobs for a given project

        :param job_list: list to hold the generated jobs
        :param project_name: the project name from the stack definition
        :param job_type: parent job - 'ci', 'autolanding' or 'rebuild'
        :param build_type: builder job - 'ci' or 'autolanding'
        :param job_config: dictionary containing the job definition
        :param job_template: template used to define the main job
        :param build_template: template used to define the build jobs
        :param job_data: supplemental data for job generation
        """

        job_base_name = get_ci_base_job_name(project_name, job_config)
        job_name = "-".join([job_base_name, job_type])
        build_list = []
        job_data['build_lookup'] = {}
        if 'configurations' in job_config:
            configurations = job_config.pop('configurations')
            for config_name in configurations:
                build_config = copy.deepcopy(job_config)
                build_config['configuration'] = config_name

                data = copy.deepcopy(configurations[config_name])
                # if the configuration doesn't contain any data it means
                # the user doesn't want any job for it
                if not data:
                    continue
                if 'template' in data:
                    template = data.pop('template')
                else:
                    template = build_template
                # A pre-defined job can be specified as a build task by
                # nam:wing the configuration to match the job name and
                # setting the template value to False or None.
                if not template or template is None:
                    build_list.append(config_name)

                else:
                    build_name = '-'.join([job_base_name, config_name,
                                           build_type])
                    build_list.append(build_name)
                    if job_type == 'rebuild':
                        # For rebuild jobs, the autolanding builder jobs will
                        # be reused. So all that is needed is the build_list.
                        continue
                    dict_union(build_config, data)
                    ctx = self.process_project_config(project_name,
                                                      build_config, job_data,
                                                      builder_job=True)
                    job_data['build_lookup'][config_name] = build_name
                    job_list.append({'name': build_name,
                                     'template': template,
                                     'ctx': ctx,
                                     'deployed': False,
                                     'parents': [job_name]})

        ctx = self.process_project_config(project_name, job_config, job_data)

        ctx['builder_list'] = ','.join(build_list)
        if job_type == 'rebuild':
            for job in job_list:
                if job['name'] in build_list:
                    job['parents'].append(job_name)
        job_list.append({'name': job_name,
                         'template': job_template,
                         'ctx': ctx,
                         'deployed': False,
                         'parents': None})
        self.master_list.append(job_list)

    def __call__(self, default_config_path, just_last_build=False):
        """Entry point for cu2d-output-ci"""
        self.default_config_path = default_config_path
        self.just_last_build = just_last_build
        set_logging(False)
        local_path = os.path.dirname(os.path.abspath(__file__))
        pocket_list = os.listdir('stacks')

        stack_list = []
        for pocket in pocket_list:
            stacks = os.listdir('stacks/{}'.format(pocket))
            for stack in stacks:
                stack_name = str(stack).split('.')[0]
                if stack_name not in ['qa']:
                    print "skipping stack {}".format(stack_name)
                    continue
                f_path = os.path.join(
                    local_path, '../stacks/{}/{}'.format(pocket, stack))
                stack_list.append({'file': f_path,
                                   'pocket': pocket,
                                   'name': stack_name})
        for stack in stack_list:
            added_job = []
            # We get the master_list from process_stack
            self.master_list = []
            default_config = load_default_cfg(default_config_path)
            stackcfg = load_stack_cfg(stack['file'], default_config)

            if not stackcfg:
                logging.error('Stack configuration failed to load. Aborting!')
                return 1
            self.process_stack([], stackcfg, None)
            for masters in self.master_list:
                for i in masters:
                    if 'builder_list' in i['ctx']:
                        builds = []
                        builds.append(i['ctx']['builder_list'])
                        param_list = i['ctx']['parameter_list']
                        for param in param_list:
                            if 'project_name' in param.name:
                                for jobs in builds:
                                    for job in jobs.split(','):
                                        if(not any(
                                                job in s for s in added_job)
                                                and not str(job).startswith(
                                                    'generic')):
                                            add.records(
                                                stack['name'],
                                                param.value,
                                                job,
                                                'coverage.xml',
                                                self.just_last_build,
                                            )

                                            added_job.append(job)
                                            print('job {}'.format(job))
