from django.db import models

import logging

from common.models import JenkinsBuild

logger = logging.getLogger("qa_dashboard")


class CoverageProduct(models.Model):
    """Ubuntu QA weekly health check 'product'.

    Examples include 'mir', 'content-hub', etc.
    """
    name = models.CharField(max_length=64)

    @property
    def last_coverage_observation(self):
        try:
            return ProductCoverageObservation.objects.filter(
                product=self).order_by('-timestamp')[0]
        except IndexError:
            return None

    def refresh_coverage_observations(self):
        ProductCoverageObservation.objects.filter(product=self).delete()
        # for each coveragedata relevant to this product, make a new observation (and compute coverages)
        coverage_projects = CoverageProject.objects.filter(product=self)
        coverage_datas = CoverageData.objects.filter(
            coverage_build__project__in=coverage_projects)
        for coverage_data in coverage_datas:
            observation = ProductCoverageObservation(
                product=self,
                timestamp=coverage_data.coverage_build.ran_at)
            observation.compute_coverage()

    @property
    def percent_reporting(self):
        coverage_projects = CoverageProject.objects.filter(product=self)
        coverage_data_exists_count = 0
        for coverage_project in coverage_projects:
            if coverage_project.last_coverage_data is not None:
                coverage_data_exists_count = coverage_data_exists_count + 1
        # TODO: this is arguably better but sqlite3 doesn't support distinct
        #coverage_datas = CoverageData.objects.filter(
        #    coverage_build__project__in=coverage_projects).distinct(
        #        'coverage_build__project')
        try:
            return float(coverage_data_exists_count) / float(len(coverage_projects))
        except ZeroDivisionError:
            # I suppose this is possible?
            return 0.0

    def __unicode__(self):
        return self.name


class CoverageProject(models.Model):
    """Our first-class object which links Jenkins to Launchpad."""
    name = models.CharField(max_length=256, unique=True)
    product = models.ManyToManyField(CoverageProduct)

    @property
    def last_build(self):
        try:
            return CoverageBuild.objects.filter(
                project=self).order_by('-ran_at')[0]
        except IndexError:
            return None

    @property
    def last_coverage_data(self):
        coverage_builds = CoverageBuild.objects.filter(
            project=self)
        try:
            return CoverageData.objects.filter(
                coverage_build__in=coverage_builds).order_by(
                    '-coverage_build__ran_at')[0]
        except IndexError:
            return None

    def __unicode__(self):
        return self.name


class CoverageBuild(JenkinsBuild):
    project = models.ForeignKey(CoverageProject)

    def __unicode__(self):
        return "{}-{}".format(self.project.name, self.ran_at)


class CoverageData(models.Model):
    url = models.CharField(max_length=4096)
    name = models.CharField(max_length=4096)
    xml_file = models.TextField()
    line_coverage = models.DecimalField(max_digits=20, decimal_places=3)
    branch_coverage = models.DecimalField(max_digits=20, decimal_places=3)
    line_count = models.IntegerField()
    hit_count = models.IntegerField()
    taken_count = models.IntegerField()
    total_count = models.IntegerField()
    coverage_build = models.ForeignKey(CoverageBuild)
    # jenkins_build
    # -- job - foreignKey Jenkinsjob
    # ---- url URLField
    # ---- name charfield
    # -- failed boolean
    # -- bugs - ManyToManyField - Bug, related_name='build_bugs'
    # ---- bug_no - charfield
    # ---- title - charfield
    # ---- status - charfield
    # ---- project - charfield
    # ---- importance -- charfield
    # ---- assignee -- charfield
    # -- ran_at
    # -- build_number - charfield
    # -- build_description - charfield
    # -- project
    # ---- name
    # ---- product
    # ------ name


class ProductCoverageObservation(models.Model):
    timestamp = models.DateTimeField()
    product = models.ForeignKey(CoverageProduct)
    line_coverage = models.DecimalField(max_digits=6,
                                        decimal_places=3,
                                        null=True,
                                        blank=True)
    branch_coverage = models.DecimalField(max_digits=6,
                                          decimal_places=3,
                                          null=True,
                                          blank=True)

    # TODO compute this on create, enforce not-null for coverage fields
    def compute_coverage(self):
        project_list = CoverageProject.objects.filter(
            product=self.product)
        hit_sum = line_sum = taken_sum = total_sum = 0
        for project in project_list:
            try:
                coverage_data = CoverageData.objects.filter(
                    coverage_build__project=project,
                    coverage_build__ran_at__lte=self.timestamp).order_by(
                        '-coverage_build__ran_at')[0]
            except IndexError:
                continue
            if coverage_data:
                line_sum += coverage_data.line_count
                hit_sum += coverage_data.hit_count
                total_sum += coverage_data.total_count
                taken_sum += coverage_data.taken_count
        try:
            self.line_coverage = float(hit_sum) / float(line_sum)
        except ZeroDivisionError:
            self.line_coverage = 0
        try:
            self.branch_coverage = float(taken_sum) / float(total_sum)
        except ZeroDivisionError:
            self.branch_coverage = 0
        self.save()
