
import datetime
from mock import patch, MagicMock

from django.test import TestCase

from gaps.util.jenkins_pull import (
    _get_artifact_urls,
    JenkinsAPIException,
    url_artifact_list,
)
from jenkinsapi import build, job
from jenkinsapi.api import Jenkins


class GetArtifactUrlsTestCase(TestCase):

    def setUp(self):
        jenkinsapi_build_mock = MagicMock(
            spec=build.Build,
            return_value=None)
        jenkinsapi_build_patch = patch.object(
            build.Build,
            '__init__',
            new=jenkinsapi_build_mock)
        jenkinsapi_build_patch.start()
        self.addCleanup(jenkinsapi_build_patch.stop)
        self.jenkinsapi_build = build.Build()

    def test_get_artifact_urls_can_match(self):
        self.jenkinsapi_build.get_artifacts = MagicMock(
            return_value=[MagicMock(url='http://foo.com/coverage.xml'), ])
        urls = _get_artifact_urls(
            self.jenkinsapi_build,
            'coverage.xml')
        self.assertEquals(['http://foo.com/coverage.xml'], urls)

    def test_get_artifact_urls_is_selective(self):
        self.jenkinsapi_build.get_artifacts = MagicMock(
            return_value=[
                MagicMock(url='http://foo.com/coverage.xml'),
                MagicMock(url='http://dungeonofdelight.com/foo'),
                MagicMock(url='http://jenkins/coverage.xml'), ])
        urls = _get_artifact_urls(
            self.jenkinsapi_build,
            'coverage.xml')
        self.assertEquals([
            'http://foo.com/coverage.xml',
            'http://jenkins/coverage.xml'], urls)

    def test_get_artifact_urls_returns_empty_list(self):
        self.jenkinsapi_build.get_artifacts = MagicMock(
            return_value=[
                MagicMock(url='http://cnn.com/this-url-has-no-match'),
                MagicMock(
                    url='http://zombo.com/match-is-not-possible'), ])
        urls = _get_artifact_urls(
            self.jenkinsapi_build,
            'coverage.xml')
        self.assertEquals([], urls)


class UrlArtifactListTestCase(TestCase):

    def setUp(self):
        super(UrlArtifactListTestCase, self).setUp()

        self.get_jenkinsapi_jenkins_mock = MagicMock(
            return_value=MagicMock(spec=Jenkins))
        get_jenkinsapi_jenkins_patch = patch(
            'gaps.util.jenkins_pull.get_jenkinsapi_jenkins',
            new=self.get_jenkinsapi_jenkins_mock)
        get_jenkinsapi_jenkins_patch.start()
        self.addCleanup(get_jenkinsapi_jenkins_patch.stop)

        self.jenkinsapi_job_mock = MagicMock(
            spec=job.Job)
        jenkinsapi_job_patch = patch(
            'jenkinsapi.job.Job',
            new=self.jenkinsapi_job_mock)
        jenkinsapi_job_patch.start()
        self.addCleanup(jenkinsapi_job_patch.stop)

        self.jenkinsapi_build_mock = MagicMock(
            spec=build.Build)
        jenkinsapi_build_patch = patch(
            'jenkinsapi.build.Build',
            new=self.jenkinsapi_build_mock)
        jenkinsapi_build_patch.start()
        self.addCleanup(jenkinsapi_build_patch.stop)

        # set up a few fake constants
        self.fake_build_ids = [1, 2, 3]
        self.fake_datetime = datetime.datetime.now()
        self.fake_baseurl = 'http://www.fake-baseurl.com'

        self.jenkinsapi_build_mock.get_timestamp = MagicMock(
            return_value=self.fake_datetime
        )
        self.jenkinsapi_job_mock.baseurl = self.fake_baseurl
        self.jenkinsapi_job_mock.get_build_ids = MagicMock(
            return_value=self.fake_build_ids
        )
        self.jenkinsapi_job_mock.get_build = MagicMock(
            return_value=self.jenkinsapi_build_mock
        )

        self.get_jenkinsapi_jenkins_mock.return_value.get_job = MagicMock(
            return_value=self.jenkinsapi_job_mock
        )

    def test_url_artifact_list_is_correct_if_no_artifact_urls(self):
        result = url_artifact_list(
            'mir-trusty-amd64-ci',
            'coverage.xml'
        )
        self.assertEqual(
            [{'url': self.fake_baseurl,
              'build_id': self.fake_build_ids[0],
              'ran_at': self.fake_datetime},
             {'url': self.fake_baseurl,
              'build_id': self.fake_build_ids[1],
              'ran_at': self.fake_datetime},
             {'url': self.fake_baseurl,
              'build_id': self.fake_build_ids[2],
              'ran_at': self.fake_datetime}],
            result)

    @patch('gaps.util.jenkins_pull._get_artifact_urls')
    def test_url_artifact_list_is_correct_if_normal_artifact_urls(
            self, get_artifact_urls_mock):
        get_artifact_urls_mock.return_value = [
            'http://foo.com/coverage.xml',
            'http://dungeonofdelight.com/foo',
            'http://jenkins/coverage.xml',
        ]
        result = url_artifact_list(
            'mir-trusty-amd64-ci',
            'coverage.xml')
        self.assertEqual(
            [{'url': self.fake_baseurl,
              'build_id': self.fake_build_ids[0],
              'ran_at': self.fake_datetime},
             {'url': self.fake_baseurl,
              'build_id': self.fake_build_ids[1],
              'ran_at': self.fake_datetime},
             {'url': self.fake_baseurl,
              'build_id': self.fake_build_ids[2],
              'ran_at': self.fake_datetime}],
            result)

    def test_return_false_if_job_not_found(self):
        self.get_jenkinsapi_jenkins_mock.return_value.get_job = MagicMock(
            return_value=self.jenkinsapi_job_mock,
            side_effect=OSError,
        )
        result = url_artifact_list(
            'mir-trusty-amd64-ci',
            'coverage.xml')
        self.assertFalse(result)

    def test_return_false_if_jenkinsapi_exception_on_get_build_ids(self):
        jenkinsapi_job = MagicMock(
            baseurl=self.fake_baseurl,
            get_build_ids=MagicMock(
                return_value=self.fake_build_ids,
                side_effect=JenkinsAPIException
            ),
            get_build=MagicMock(return_value=self.jenkinsapi_build_mock)
        )
        self.get_jenkinsapi_jenkins_mock.return_value.get_job = MagicMock(
            return_value=jenkinsapi_job
        )
        result = url_artifact_list(
            'mir-trusty-amd64-ci',
            'coverage.xml'
        )
        self.assertFalse(result)
