from django.test import TestCase

from gaps.models import (
    CoverageBuild,
    CoverageData,
    CoverageProject,
    CoverageProduct,
)
from gaps.util import add

import datetime
import decimal
import mock
from os import path


def make_decimal(number):
    c = decimal.getcontext()
    c.prec = 3
    dec = c.create_decimal(number)
    return dec


class TestJenkinsPullCoverage(TestCase):
    def setUp(self):
        self.url_list = [
            {
                'url': 'http://jenkins.fake/job/libqtdbustest-trusty'
                '-i386-ci/11/artifact/work/results/coverage.xml',
                'build_id': 11,
                'ran_at': datetime.datetime(
                    2014, 4, 18, 17, 31, 24)
            },
            {
                'url': 'http://jenkins.fake/job/libqtdbustest-trusty'
                '-i386-ci/10/artifact/work/results/coverage.xml',
                'build_id': 10,
                'ran_at': datetime.datetime(
                    2014, 3, 18, 17, 31, 24)
            }

        ]

        localdir = path.dirname(__file__)
        with open(path.join(localdir, 'data/coverage.xml'), 'r') as f:
            jenkins_get_data = f.read()
        f.close()
        self.jdata = jenkins_get_data

    def test_jenkins_get_data_from_common_exists(self):
        """Verify jenkins_get method is in common.managment"""
        from common.management import jenkins_get  # noqa

    @mock.patch('gaps.util.add.jenkins_get')
    @mock.patch('gaps.util.add.jenkins_pull')
    def test_jenkins_pull_coverage_add_to_database(self, mock_pull, mock_get):
        mock_get.return_value = self.jdata
        mock_pull.url_artifact_list.return_value = self.url_list
        add.records(
            'qa',
            'libqtdbustest',
            'libqtdbustest-trusty-amd64-ci',
            'coverage.xml',
        )

        # There must be 2 records when getting all the builds
        self.assertEquals(
            CoverageData.objects.count(),
            2
        )

        data_result = CoverageData.objects.filter(
            url__contains='coverage.xml').latest('coverage_build')

        self.assertEquals(
            data_result.url,
            'http://jenkins.fake/job/libqtdbustest-trusty-i386-ci'
            '/11/artifact/work/results/coverage.xml'
        )
        self.assertEquals(data_result.name, 'coverage.xml')
        self.assertEquals(data_result.xml_file, '')

        self.assertEquals(data_result.branch_coverage, make_decimal(0.514))
        self.assertEquals(data_result.line_coverage, make_decimal(.884))
        self.assertEquals(data_result.hit_count, 107)
        self.assertEquals(data_result.line_count, 121)
        self.assertEquals(data_result.taken_count, 75)
        self.assertEquals(data_result.total_count, 146)

        build_result = CoverageBuild.objects.filter(
            build_number='11').latest('ran_at')

        self.assertEquals(str(build_result.build_number), '11')
        self.assertEqual(build_result.build_description, 'None')

        project_result = CoverageProject.objects.filter(
            name='libqtdbustest').latest('name')

        self.assertEquals(project_result.name, 'libqtdbustest')

        product_result = CoverageProduct.objects.filter(
            name='qa').latest('name')

        self.assertEquals(product_result.name, 'qa')

    @mock.patch('gaps.util.add.jenkins_get')
    @mock.patch('gaps.util.add.jenkins_pull')
    def test_jenkins_pull_coverage_last_add_to_database(self,
                                                        mock_pull,
                                                        mock_get):
        mock_get.return_value = self.jdata
        mock_pull.url_artifact_list.return_value = self.url_list
        add.records(
            'qa',
            'libqtdbustest',
            'libqtdbustest-trusty-amd64-ci',
            'coverage.xml',
            True
        )
        # There must be only one record when getting the last build
        self.assertEquals(
            CoverageData.objects.count(),
            1
        )

        data_result = CoverageData.objects.filter(
            url__contains='coverage.xml').latest('coverage_build')

        self.assertEquals(
            data_result.url,
            'http://jenkins.fake/job/libqtdbustest-trusty-i386-ci'
            '/11/artifact/work/results/coverage.xml'
        )
        self.assertEquals(data_result.name, 'coverage.xml')
        self.assertEquals(data_result.xml_file, '')
        self.assertEquals(data_result.branch_coverage, make_decimal(0.514))
        self.assertEquals(data_result.line_coverage, make_decimal(0.884))
        self.assertEquals(data_result.hit_count, 107)
        self.assertEquals(data_result.line_count, 121)
        self.assertEquals(data_result.taken_count, 75)
        self.assertEquals(data_result.total_count, 146)

        build_result = CoverageBuild.objects.filter(
            build_number='11').latest('ran_at')

        self.assertEquals(str(build_result.build_number), '11')
        self.assertEqual(build_result.build_description, 'None')

        project_result = CoverageProject.objects.filter(
            name='libqtdbustest').latest('name')

        self.assertEquals(project_result.name, 'libqtdbustest')

        product_result = CoverageProduct.objects.filter(
            name='qa').latest('name')

        self.assertEquals(product_result.name, 'qa')
