
import logging
import os
import subprocess

from gaps.util.cu2d.cupstream2distro_config.c2dconfigutils.cu2dUpdateCi import UpdateCi  # noqa
from gaps.util.cu2d.cupstream2distro_config.c2dconfigutils.c2dconfigutils import (
    load_default_cfg,
    load_stack_cfg,
)
from qa_dashboard.settings import CU2D_LP_REPO

logger = logging.getLogger('qa_dashboard')


def refresh_cupstream2distro_config_repo(cu2d_config_repo_dir):
    """Refresh our local lp:cupstream2distro-config repo.

    NOTE: also establish the repo if it's not found.

    :param cu2d_config_repo_dir: path to the repo

    """
    if os.path.exists(cu2d_config_repo_dir):
        with os.chdir(cu2d_config_repo_dir):
            subprocess.call(['bzr', 'pull'])
    else:
        (enclosing_dir, repo_name) = os.path.split(cu2d_config_repo_dir)
        with os.chdir(enclosing_dir):
            subprocess.call(
                ['bzr',
                 'branch',
                 "lp:{}".format(CU2D_LP_REPO),
                 repo_name]
            )
            # touch __init__.py here so that we can treat as a module ;)
            open('__init__.py', 'a').close()


def get_stack_list_from_filesystem(stacks_dir):
    """Construct a list of stacks from cu2d *.cfg files in the filesystem.

    :param stacks_dir: dir containing stacks (<repo>/stacks typically)
    :returns: list of dicts: 'config_filepath', 'pocket', and 'name' for stack
    """
    stack_list = []
    pocket_dirs = os.listdir(stacks_dir)
    for pocket_dir in pocket_dirs:
        stack_config_filenames = os.listdir(
            os.path.join(
                stacks_dir,
                pocket_dir
            )
        )
        for stack_config_filename in stack_config_filenames:
            (stack_name, _) = os.path.splitext(stack_config_filename)
            stack_config_filepath = os.path.join(
                stacks_dir,
                pocket_dir,
                stack_config_filename
            )
            stack_list.append({'config_filepath': stack_config_filepath,
                               'pocket': pocket_dir,
                               'name': stack_name})
    return stack_list


def generate_job_configs_from_stack_config(
        default_config_path,
        stack,
        project_name=None):
    """Given a stack config file, use cu2d to generate a list of job configs.

    :param default_config_path: dir containing 'default.cfg' (<repo>/ci ATM)
    :param stack: 'config_filepath', 'pocket', 'name' dict representing stack
    :param project_name: Launchpad name; generate only relevant configs
    """
    # yes this is an object
    update_ci = UpdateCi()
    # this is necessary due to a deeply buried reference in _get_build_script
    update_ci.default_config_path = default_config_path
    logger.debug("Generating jobs for stack {} for project {} from {}".format(
        stack, project_name, default_config_path))
    default_config = load_default_cfg(default_config_path)
    stack_cfg = load_stack_cfg(stack['config_filepath'], default_config)
    if not stack_cfg:
        logging.error('Stack configuration failed to load. Aborting!')
        return
    job_configs = []
    try:
        update_ci.process_stack(job_configs, stack_cfg, target_project=project_name)
    # TODO: seeing various and sundry errors, nail down then differentiate
    except TypeError, e:
        # this happens when the 'projects' field in the config is empty
        pass
    except Exception, e:
        logger.error(
            "{}: failed to process stack {}: {}.".format(
                type(e), stack, e
            )
        )
    return job_configs


def generate_jenkins_job_names(
        default_config_path,
        stacks_dir,
        project_name=None):
    """Use cu2d tools to generate Jenkins job names.

    :param default_config_path: dir containing 'default.cfg' (<repo>/ci ATM)
    :param stacks_dir: directory containing stacks from which to generate
    :param project_name: Launchpad name; return only relevant names
    :returns: a list of Jenkins job names
    """

    stack_config_list = get_stack_list_from_filesystem(stacks_dir)
    job_config_list = []
    for stack_config in stack_config_list:
        job_config_list.extend(
            generate_job_configs_from_stack_config(
                default_config_path,
                stack_config,
                project_name
            )
        )
    job_names = [job_config['name'] for job_config in job_config_list]
    return job_names
