# QA Dashboard
# Copyright 2012-2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lxml import etree

from common.management import jenkins_get

"""Classes and utilities to extract QA data from jenkins"""


def get_text_from_url(url):
    response = jenkins_get(url, as_json=False)
    if response:
        return response
    raise EnvironmentError("No data at %s" % (url))


class JenkinsCoverage(object):

    """Provides methods and attributes for a jenkins code coverage report."""

    def __init__(self, url, data=None):
        """Creates the coverage object"""
        super(JenkinsCoverage, self).__init__()
        self.url = url
        self.data = data
        if self.data is None:
            self.data = get_text_from_url(self.url)
        self.root = etree.XML(self.data)
        self.get_counts()

    def __str__(self):
        """Provides a string representation of the xml tree"""
        return "<JenkinsCoverage %s>" % (self.data)

    @property
    def line_coverage(self):
        try:
            return float(self.hits) / float(self.lines)
        except ZeroDivisionError:
            return 0

    @property
    def branch_coverage(self):
        try:
            return float(self.taken) / float(self.total)
        except ZeroDivisionError:
            return 0

    def get_attribute(self, attribute, root=None):
        if root is None:
            root = self.root
        return root.get(attribute)

    def get_coverage_from_xml(self):
        coverage = dict()
        for attr in ['line-rate', 'branch-rate']:
            coverage[attr] = self.get_attribute(attr)
        return coverage

    def get_packages(self):
        packages = list()
        for element in self.root[1]:
            packages.append(self.get_attribute('name', element))
        return packages

    def get_counts(self, package=None):
        """Records detailed coverage stats from the coverage xml"""
        branch_count = 0
        hit_count = 0
        line_count = 0
        taken_count = 0
        total_count = 0
        collect = True
        for element in self.root.iter():
            if package is not None and element.tag == 'package':
                if self.get_attribute('name', element) == package:
                    collect = True
                else:
                    collect = False
            if collect and element.tag == 'line':
                line_count += 1
                if self.get_attribute('hits', element) != '0':
                    hit_count += 1
                if self.get_attribute('branch', element) == 'true':
                    branch_count += 1
                    coverage = self.get_attribute('condition-coverage',
                                                  element)
                    coverage = coverage[coverage.find('(') + 1:-1]
                    (taken, total) = coverage.split('/')
                    taken_count += int(taken)
                    total_count += int(total)
        if package is not None:
            line_rate = float(0)
            branch_rate = float(0)
            try:
                line_rate = float(hit_count) / float(line_count)
            except ZeroDivisionError:
                line_rate = float(0)
            try:
                branch_rate = float(taken_count) / float(total_count)
            except ZeroDivisionError:
                branch_rate = float(0)
            package_dict = dict(
                name=package,
                lines=line_count,
                hits=hit_count,
                branches=branch_count,
                taken=taken_count,
                total=total_count,
                line_rate=line_rate,
                branch_rate=branch_rate
            )
            return type('JenkinsCoveragePackage', (object,), package_dict)
        self.lines = line_count
        self.hits = hit_count
        self.branches = branch_count
        self.taken = taken_count
        self.total = total_count
        return self
