
import logging
import time

from jenkinsapi.api import Jenkins
from jenkinsapi.custom_exceptions import JenkinsAPIException

from gaps.util.cu2d import generate_jenkins_job_names
from qa_dashboard.settings import (
    JENKINS_URL,
    CU2D_DEFAULT_CONFIG_DIR,
    CU2D_STACKS_DIR,
)

logger = logging.getLogger('qa_dashboard')


def get_jenkinsapi_jenkins():
    """Log into Jenkins (if necessary), return our jenkinsapi.Jenkins."""
    if not getattr(get_jenkinsapi_jenkins, '_api', False):
        try:
            logger.info("Initializing jenkinsapi.")
            setattr(get_jenkinsapi_jenkins, '_api', Jenkins(JENKINS_URL))
        except:
            logger.debug("Failed to initialize jenkinsapi, trying again.")
            time.sleep(5)
            setattr(get_jenkinsapi_jenkins, '_api', Jenkins(JENKINS_URL))
    return getattr(get_jenkinsapi_jenkins, '_api', None)


def _get_artifact_urls(build, artifact_name):
    """Return artifact urls from build

    :param build: Jenkins build to get artifacts from
    :param artifact_name: Name of the artifact to get the url from
    """
    urls = [archive.url for archive in build.get_artifacts()
            if artifact_name in archive.url]
    return urls


def url_artifact_list(jenkins_job_name,
                      artifact_name,
                      last_build=0,
                      just_last_build=False):
    """
    gets a specific artifact for all successful builds from
    specified job_name

    :param string jenkinks_job_name: the name of the job with the artifact
    :param string artifact_name: the file name of the artifact in jenkins
    :param integer last_build: the last build already stored in the database
    use this to return a list of only artifacts that are not in the database
    :param just_last_build: get just the last build?

    example:
    >>>jenkins_pull.url_list(
        'kubuntu-packagers-kubuntu-packaging-qtbase-opensource-src-trusty-'
        'armhf-ci',
        'coverage.xml')

    :return: List of urls
    """
    jenkapi = get_jenkinsapi_jenkins()
    try:
        job = jenkapi.get_job(jenkins_job_name)
    except:
        logger.debug(
            "Job with name {} does not exist.".format(jenkins_job_name))
        return False
    try:
        build_ids = job.get_build_ids()
    except JenkinsAPIException, e:
        logger.debug("JenkinsAPIException: {}.".format(e))
        return False
    if just_last_build:
        build_ids = [list(build_ids)[-1]]
    url_list = []
    for build_id in build_ids:
        if int(last_build) < int(build_id):
            try:
                build = job.get_build(build_id)
            except:
                continue
            ran_at = build.get_timestamp()

            urls = None
            if build.get_status() == 'SUCCESS':
                urls = _get_artifact_urls(build, artifact_name)

            if urls:
                for url in urls:
                    logger.debug("adding url {}".format(url))
                    url_list.append({
                        'url': ''.join(url),
                        'build_id': build_id,
                        'ran_at': ran_at
                        })
            else:
                url_list.append({
                    'url': job.baseurl,
                    'build_id': build_id,
                    'ran_at': ran_at
                })
    logger.debug(url_list)
    if url_list:
        return url_list
    else:
        return False
